/*
 *	File:	calendar.c
 *	Date:	10.01.2011
 */

#include "main.h"

const uint8_t month_day[] =
{ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

const uint16_t week_day[] =
{ 0x4263, 0xA8BD, 0x42BF, 0x4370, 0xABBF, 0xA8BF, 0x43B2};

//-----------------------------------------------------------------------------
uint8_t calendar_week_day(uint16_t year, uint8_t month, uint8_t day)
{
	uint16_t tmp1, tmp2, tmp3, tmp4, week_day;

	if (month < 3)
	{
		month = month + 12;
		year = year - 1;
	}

	tmp1 = (6 * (month + 1)) / 10;
	tmp2 = year / 4;
	tmp3 = year / 100;
	tmp4 = year / 400;
	week_day = day + (2 * month) + tmp1 + year + tmp2 - tmp3 + tmp4 + 1;
	week_day = week_day % 7;

	return (week_day);
}

//-----------------------------------------------------------------------------
uint32_t calendar_coder(calendar_type * calendar)
{
	uint32_t tmp;
	uint8_t i;

	tmp = calendar->year * (365 * 24 * 3600) + (calendar->year / 4) * 24 * 3600;

	for (i = 0; i < calendar->month - 1; i++)
	{
		tmp += 24 * 3600 * month_day[0];
	}

	tmp += calendar->day * 24 * 3600;
	tmp += calendar->hour * 3600;
	tmp += calendar->minutes * 60;

	return tmp + calendar->seconds;
}

//-----------------------------------------------------------------------------
void calendar_decoder(uint32_t value, calendar_type * calendar)
{
	int16_t tmp;

	calendar->year = value / (365 * 24 * 3600);

	tmp = ((value % (365 * 24 * 3600)) / (24 * 3600)) - (calendar->year / 4);
	if ((tmp <= 0) && (calendar->year > 3))
	{
		calendar->year--;
		if (calendar_check_leap(calendar->year + 2000))
		{
			tmp += 366;
		}
		else
		{
			tmp += 365;
		}
	}

	calendar->month = 0;
	while (tmp > 0)
	{
		calendar->day = tmp;
		if (calendar->month++ == 2)
		{
			if (calendar_check_leap(calendar->year + 2000))
				tmp -= month_day[calendar->month - 1] + 1;
			else
				tmp -= month_day[calendar->month - 1];
		}
		else
		{
			tmp -= month_day[calendar->month - 1];
		}
	}

	calendar->day_of_the_week = calendar_week_day(2000 + calendar->year,
			calendar->month, calendar->day);

	calendar->hour = ((value % (365 * 24 * 3600)) % (24 * 3600)) / 3600;
	calendar->minutes = (((value % (365 * 24 * 3600)) % (24 * 3600)) % 3600)
			/ 60;
	calendar->seconds = (((value % (365 * 24 * 3600)) % (24 * 3600)) % 3600)
			% 60;
}

//-----------------------------------------------------------------------------
uint8_t calendar_check_leap(uint16_t year)
{
	if ((year % 400) == 0)
	{
		return TRUE;
	}
	else if ((year % 100) == 0)
	{
		return FALSE;
	}
	else if ((year % 4) == 0)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

